@@ -130,7 +130,7 @@ class ConfManager(LoggerMixin, threading.Thread):
             self.log_info("Stopping pending Ansible process.")
             self.ansible_process.terminate()
 
-    def wait_all_vm_ips(self, timeout=Config.ANSIBLE_INSTALL_TIMEOUT):
+    def wait_all_vm_ips(self, timeout=Config.WAIT_PUBLIC_IP_TIMEOUT):
         """
         Assure that all the VMs of the Inf. have all the requested public IPs assigned
         """
@@ -159,8 +159,10 @@ class ConfManager(LoggerMixin, threading.Thread):
                 time.sleep(Config.CONFMAMAGER_CHECK_STATE_INTERVAL)
 
         if not success:
+            self.inf.set_configured(False)
             self.log_warn("Error waiting all the VMs to have all the requested IPs")
         else:
+            self.inf.set_configured(True)
             self.log_info("All the VMs have all the requested IPs")
             # do a final update of all VMs
             for vm in self.inf.get_vm_list():
@@ -355,6 +357,7 @@ class ConfManager(LoggerMixin, threading.Thread):
         """
         Launch the ctxt agent to configure the specified tasks in the specified VM
         """
+        ssh = None
         pid = None
         tmp_dir = None
         try:
@@ -376,7 +379,7 @@ class ConfManager(LoggerMixin, threading.Thread):
                 self.log_info("Copy the contextualization agent config file")
 
                 # Copy the contextualization agent config file
-                ssh = vm.get_ssh_ansible_master()
+                ssh = vm.get_ssh_ansible_master(auto_close=False)
                 ssh.sftp_mkdir(remote_dir)
                 ssh.sftp_put(conf_file, remote_dir + "/" + os.path.basename(conf_file))
 
@@ -391,11 +394,11 @@ class ConfManager(LoggerMixin, threading.Thread):
                     vault_password = vm.info.systems[0].getValue("vault.password")
                     if vault_password:
                         vault_export = "export VAULT_PASS='%s' && " % vault_password
-                    (pid, _, _) = ssh.execute(vault_export + "nohup python " + Config.REMOTE_CONF_DIR + "/" +
-                                              str(self.inf.id) + "/" + ctxt_agent_command +
+                    (pid, _, _) = ssh.execute("nohup sh -c \"" + vault_export + "python3 " + Config.REMOTE_CONF_DIR +
+                                              "/" + str(self.inf.id) + "/" + ctxt_agent_command +
                                               Config.REMOTE_CONF_DIR + "/" + str(self.inf.id) + "/" +
                                               "/general_info.cfg " + remote_dir + "/" + os.path.basename(conf_file) +
-                                              " > " + remote_dir + "/stdout" + " 2> " + remote_dir +
+                                              "\" > " + remote_dir + "/stdout" + " 2> " + remote_dir +
                                               "/stderr < /dev/null & echo -n $!")
 
                     self.log_info("Ansible process to configure " + str(vm.im_id) + " launched with pid: " + pid)
@@ -408,6 +411,8 @@ class ConfManager(LoggerMixin, threading.Thread):
             pid = None
             self.log_exception("Error launching the ansible process to configure VM with ID %s" % str(vm.im_id))
         finally:
+            if ssh:
+                ssh.close()
             if tmp_dir:
                 shutil.rmtree(tmp_dir, ignore_errors=True)
 
@@ -534,6 +539,8 @@ class ConfManager(LoggerMixin, threading.Thread):
                 node_line += ' IM_NODE_NUM=' + str(vm.im_id)
                 node_line += ' IM_NODE_VMID=' + str(vm.id)
                 node_line += ' IM_NODE_CLOUD_TYPE=' + vm.cloud.type
+                if vm.cloud.server:
+                    node_line += ' IM_NODE_CLOUD_SERVER=' + vm.cloud.server
                 node_line += ifaces_im_vars
 
                 for app in vm.getInstalledApplications():
@@ -657,7 +664,7 @@ class ConfManager(LoggerMixin, threading.Thread):
                 if disk_mount_path and disk_fstype:
                     # This recipe works with EC2, OpenNebula and Azure. It must be
                     # tested/completed with other providers
-                    res += '  - include: utils/tasks/disk_format_mount.yml\n'
+                    res += '  - include_tasks: utils/tasks/disk_format_mount.yml\n'
                     res += '    vars:\n'
                     res += '      device: "/dev/{{item.key}}"\n'
                     res += '      mount_path: "' + disk_mount_path + '"\n'
@@ -667,7 +674,11 @@ class ConfManager(LoggerMixin, threading.Thread):
                     # Devices hdb, sdb, xvdb, etc
                     res += "item.key.endswith('d%s') or " % disk_device[-1]
                     # Devices nvme0n1 (NVMe type in EC2)
-                    res += "item.key.startswith('nvme%sn1')" % (ord(disk_device[-1]) - 97)
+                    res += "item.key.startswith('nvme%sn1') or " % (ord(disk_device[-1]) - 97)
+                    # Full name device
+                    res += "item.key == '%s' or " % disk_device
+                    # Use also link names (Linode case)
+                    res += "'%s' in item.value.links.ids" % disk_device
                     res += ")\n"
 
             cont += 1
@@ -689,7 +700,7 @@ class ConfManager(LoggerMixin, threading.Thread):
 
         conf_content += "  pre_tasks: \n"
         # Basic tasks set copy /etc/hosts ...
-        conf_content += "  - include: utils/tasks/main.yml\n"
+        conf_content += "  - include_tasks: utils/tasks/main.yml\n"
 
         conf_content += "  tasks: \n"
         conf_content += "  - debug: msg='Install user requested apps'\n"
@@ -753,8 +764,7 @@ class ConfManager(LoggerMixin, threading.Thread):
         """
         recipe_files = []
 
-        conf_filename = tmp_dir + "/" + ctxt_elem.configure + \
-            "_" + ctxt_elem.system + "_task.yml"
+        conf_filename = tmp_dir + "/" + ctxt_elem.configure + "_" + ctxt_elem.system + "_task.yml"
         if not os.path.isfile(conf_filename):
             configure = self.inf.radl.get_configure_by_name(ctxt_elem.configure)
             conf_content = self.add_ansible_header(vm.getOS().lower())
@@ -762,16 +772,16 @@ class ConfManager(LoggerMixin, threading.Thread):
             if vault_password:
                 vault_edit = self.get_vault_editor(vault_password)
                 if configure.recipes.strip().startswith("$ANSIBLE_VAULT"):
-                    recipes = vault_edit.vault.decrypt(configure.recipes.strip())
+                    recipes = vault_edit.vault.decrypt(configure.recipes.strip()).decode()
                 else:
                     recipes = configure.recipes
                 conf_content = merge_recipes(conf_content, recipes)
-                conf_content = vault_edit.vault.encrypt(conf_content)
+                conf_content = vault_edit.vault.encrypt(conf_content).decode()
             else:
                 conf_content = merge_recipes(conf_content, configure.recipes)
 
             conf_out = open(conf_filename, 'w')
-            conf_out.write(conf_content)
+            conf_out.write(str(conf_content))
             conf_out.close()
             recipe_files.append(ctxt_elem.configure + "_" + ctxt_elem.system + "_task.yml")
 
@@ -795,6 +805,21 @@ class ConfManager(LoggerMixin, threading.Thread):
         """
         success = True
         tmp_dir = None
+
+        if self.inf.ansible_configured:
+            # Check that remote_dir exists
+            try:
+                remote_dir = Config.REMOTE_CONF_DIR + "/" + str(self.inf.id) + "/"
+                ssh = self.inf.vm_master.get_ssh(retry=True)
+                files = ssh.sftp_list(remote_dir)
+            except Exception:
+                self.log_exception("Error listing remote dir %s." % remote_dir)
+                files = []
+            # if there are no files, reconfigure ansible
+            if len(files) < 4:
+                self.log_warn("Remote dir %s not found. Reinstall ansible on master node." % remote_dir)
+                self.inf.ansible_configured = False
+
         if not self.inf.ansible_configured:
             success = False
             cont = 0
@@ -802,6 +827,7 @@ class ConfManager(LoggerMixin, threading.Thread):
                 self.log_info("Sleeping %s secs." % (cont ** 2 * 5))
                 time.sleep(cont ** 2 * 5)
                 cont += 1
+                ssh = None
                 try:
                     self.log_info("Start the contextualization process.")
 
@@ -810,7 +836,7 @@ class ConfManager(LoggerMixin, threading.Thread):
                     else:
                         if not self.inf.vm_master:
                             raise Exception("No master VM found.")
-                        ssh = self.inf.vm_master.get_ssh(retry=True)
+                        ssh = self.inf.vm_master.get_ssh(retry=True, auto_close=False)
                         if not ssh:
                             raise Exception("Master VM does not have IP.")
                         # Activate tty mode to avoid some problems with sudo in
@@ -822,7 +848,11 @@ class ConfManager(LoggerMixin, threading.Thread):
                         tmp_dir = tempfile.mkdtemp()
                         # Now call the ansible installation process on the
                         # master node
-                        configured_ok = self.configure_ansible(ssh, tmp_dir)
+                        ansible_version = None
+                        if self.inf.radl.contextualize.options:
+                            if 'ansible_version' in self.inf.radl.contextualize.options:
+                                ansible_version = self.inf.radl.contextualize.options['ansible_version'].getValue()
+                        configured_ok = self.configure_ansible(ssh, tmp_dir, ansible_version)
 
                         if not configured_ok:
                             self.log_error("Error in the ansible installation process")
@@ -882,6 +912,8 @@ class ConfManager(LoggerMixin, threading.Thread):
                         self.inf.ansible_configured = False
                     success = False
                 finally:
+                    if ssh:
+                        ssh.close()
                     if tmp_dir:
                         shutil.rmtree(tmp_dir, ignore_errors=True)
 
@@ -915,6 +947,7 @@ class ConfManager(LoggerMixin, threading.Thread):
         success = True
         if not self.inf.ansible_configured:
             # Select the master VM
+            ssh = None
             try:
                 self.inf.add_cont_msg("Select master VM")
                 self.inf.select_vm_master()
@@ -951,7 +984,7 @@ class ConfManager(LoggerMixin, threading.Thread):
 
                 # Check and change if necessary the credentials of the master
                 # vm
-                ssh = self.inf.vm_master.get_ssh(retry=True)
+                ssh = self.inf.vm_master.get_ssh(retry=True, auto_close=False)
                 # Activate tty mode to avoid some problems with sudo in REL
                 ssh.tty = True
                 self.change_master_credentials(ssh)
@@ -963,6 +996,9 @@ class ConfManager(LoggerMixin, threading.Thread):
             except Exception:
                 self.log_exception("Error waiting the master VM to be running")
                 self.inf.set_configured(False)
+            finally:
+                if ssh:
+                    ssh.close()
         else:
             self.inf.set_configured(True)
 
@@ -986,8 +1022,7 @@ class ConfManager(LoggerMixin, threading.Thread):
             if self.inf.radl.configures:
                 for elem in self.inf.radl.configures:
                     if elem is not None and not os.path.isfile(tmp_dir + "/" + elem.name + ".yml"):
-                        conf_out = open(
-                            tmp_dir + "/" + elem.name + ".yml", 'w')
+                        conf_out = open(tmp_dir + "/" + elem.name + ".yml", 'w')
                         conf_out.write(elem.recipes)
                         conf_out.write("\n\n")
                         conf_out.close()
@@ -1016,8 +1051,7 @@ class ConfManager(LoggerMixin, threading.Thread):
                 for ctxt_elem in contextualizes[ctxt_num]:
                     if ctxt_elem.system in vm_group and ctxt_elem.get_ctxt_tool() == "Ansible":
                         vm = vm_group[ctxt_elem.system][0]
-                        filenames.extend(self.generate_playbook(
-                            vm, ctxt_elem, tmp_dir))
+                        filenames.extend(self.generate_playbook(vm, ctxt_elem, tmp_dir))
 
             filenames.append(self.generate_etc_hosts(tmp_dir))
             filenames.append(self.generate_inventory(tmp_dir))
@@ -1035,8 +1069,7 @@ class ConfManager(LoggerMixin, threading.Thread):
             if self.inf.radl.ansible_hosts:
                 for ansible_host in self.inf.radl.ansible_hosts:
                     (user, passwd, private_key) = ansible_host.getCredentialValues()
-                    ssh = SSHRetry(ansible_host.getHost(),
-                                   user, passwd, private_key)
+                    ssh = SSHRetry(ansible_host.getHost(), user, passwd, private_key)
                     ssh.sftp_mkdir(remote_dir)
                     ssh.sftp_put_files(recipe_files)
             else:
@@ -1062,7 +1095,7 @@ class ConfManager(LoggerMixin, threading.Thread):
            - timeout(int): Max time to wait the VM to be running.
         Returns: True if all the VMs are running or false otherwise
         """
-        delay = 10
+        delay = Config.CHECK_CTXT_PROCESS_INTERVAL
         wait = 0
         while not self._stop_thread and wait < timeout:
             if not vm.destroy:
@@ -1179,7 +1212,7 @@ class ConfManager(LoggerMixin, threading.Thread):
                     # passwd value
                     if passwd and new_passwd:
                         self.log_info("Changing password to master VM")
-                        (out, err, code) = ssh.execute('echo "' + passwd + '" | sudo -S bash -c \'echo "' +
+                        (out, err, code) = ssh.execute('echo "' + passwd + '" | sudo -S sh -c \'echo "' +
                                                        user + ':' + new_passwd +
                                                        '" | /usr/sbin/chpasswd && echo "OK"\' 2> /dev/null')
 
@@ -1314,12 +1347,12 @@ class ConfManager(LoggerMixin, threading.Thread):
         conf_all_out = open(tmp_dir + "/" + all_filename, 'w')
         conf_all_out.write("---\n")
         conf_all_out.write("- hosts: " + group + "\n")
-        conf_all_out.write("- include: " + filename + ".yml\n")
+        conf_all_out.write("- import_playbook: " + filename + ".yml\n")
         conf_all_out.write("\n\n")
         conf_all_out.close()
         return all_filename
 
-    def configure_ansible(self, ssh, tmp_dir):
+    def configure_ansible(self, ssh, tmp_dir, ansible_version=None):
         """
         Install ansible in the master node
 
@@ -1408,9 +1441,18 @@ class ConfManager(LoggerMixin, threading.Thread):
             except Exception:
                 self.log_exception("Error removing requiretty. Ignoring.")
 
-            self.inf.add_cont_msg("Configure Ansible in the master VM.")
-            self.log_info("Call Ansible to (re)configure in the master node")
+            ver_msg = " (v. %s)" % ansible_version if ansible_version else ""
+            self.inf.add_cont_msg("Configure Ansible%s in the master VM." % ver_msg)
+            self.log_info("Call Ansible%s to (re)configure in the master node" % ver_msg)
+            ansible_version_env = None
+            if ansible_version:
+                # Set ansible verion env var
+                ansible_version_env = os.getenv('ANSIBLE_VERSION')
+                os.environ['ANSIBLE_VERSION'] = ansible_version
             (success, msg) = self.call_ansible(tmp_dir, "inventory.cfg", ConfManager.MASTER_YAML, ssh)
+            if ansible_version_env:
+                # restore original value
+                os.environ['ANSIBLE_VERSION'] = ansible_version_env
 
             if not success:
                 self.log_error("Error configuring master node: " + msg + "\n\n")
