@@ -305,6 +305,17 @@ def format_output(res, default_type="text/plain", field_name=None, list_field_na
     return info
 
 
+@app.hook('after_request')
+def enable_cors():
+    """
+    Enable CORS to javascript SDK
+    """
+    if Config.ENABLE_CORS:
+        bottle.response.headers['Access-Control-Allow-Origin'] = Config.CORS_ORIGIN
+        bottle.response.headers['Access-Control-Allow-Methods'] = 'PUT, GET, POST, DELETE, OPTIONS'
+        bottle.response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, Authorization'
+
+
 @app.route('/infrastructures/:infid', method='DELETE')
 def RESTDestroyInfrastructure(infid=None):
     try:
@@ -1020,6 +1031,11 @@ def RESTRebootVM(infid=None, vmid=None):
         return return_error(400, "Error rebooting VM: %s" % get_ex_error(ex))
 
 
+@app.route("/<url:re:.+>", method='OPTIONS')
+def ReturnOptions(**kwargs):
+    return {}
+
+
 @app.route('/version', method='GET')
 def RESTGeVersion():
     try:
@@ -1072,6 +1088,46 @@ def RESTCreateDiskSnapshot(infid=None, vmid=None, disknum=None):
         return return_error(400, "Error creating snapshot: %s" % get_ex_error(ex))
 
 
+def _filters_str_to_dict(filters_str):
+    filters = {}
+    for elem in filters_str.split(","):
+        kv = elem.split("=")
+        if len(kv) != 2:
+            raise Exception("Incorrect format")
+        else:
+            filters[kv[0]] = kv[1]
+    return filters
+
+
+@app.route('/clouds/:cloudid/:param', method='GET')
+def RESTGetCloudInfo(cloudid=None, param=None):
+    try:
+        auth = get_auth_header()
+    except Exception:
+        return return_error(401, "No authentication data provided")
+
+    try:
+        if param == 'images':
+            filters = None
+            if "filters" in bottle.request.params.keys():
+                try:
+                    filters = _filters_str_to_dict(bottle.request.params.get("filters"))
+                except Exception:
+                    return return_error(400, "Invalid format in filters parameter.")
+            images = InfrastructureManager.GetCloudImageList(cloudid, auth, filters)
+            bottle.response.content_type = "application/json"
+            return format_output(images, default_type="application/json", field_name="images")
+        elif param == 'quotas':
+            quotas = InfrastructureManager.GetCloudQuotas(cloudid, auth)
+            bottle.response.content_type = "application/json"
+            return format_output(quotas, default_type="application/json", field_name="quotas")
+    except InvaliddUserException as ex:
+        return return_error(401, "Error getting cloud info: %s" % get_ex_error(ex))
+    except Exception as ex:
+        logger.exception("Error getting cloud info")
+        return return_error(400, "Error getting cloud info: %s" % get_ex_error(ex))
+
+
 @app.error(403)
 def error_mesage_403(error):
     return return_error(403, error.body)
