@@ -53,9 +53,10 @@ class Config:
     MAX_VM_FAILS = 3
     WAIT_RUNNING_VM_TIMEOUT = 1800
     WAIT_SSH_ACCCESS_TIMEOUT = 300
+    WAIT_PUBLIC_IP_TIMEOUT = 90
     XMLRCP_PORT = 8899
     XMLRCP_ADDRESS = "0.0.0.0"
-    ACTIVATE_REST = False
+    ACTIVATE_REST = True
     REST_PORT = 8800
     REST_ADDRESS = "0.0.0.0"
     USER_DB = ""
@@ -85,7 +86,7 @@ class Config:
     MAX_SSH_ERRORS = 5
     PRIVATE_NET_MASKS = ["10.0.0.0/8", "172.16.0.0/12", "192.168.0.0/16",
                          "169.254.0.0/16", "100.64.0.0/10", "192.0.0.0/24", "198.18.0.0/15"]
-    CHECK_CTXT_PROCESS_INTERVAL = 5
+    CHECK_CTXT_PROCESS_INTERVAL = 10
     CONFMAMAGER_CHECK_STATE_INTERVAL = 5
     UPDATE_CTXT_LOG_INTERVAL = 20
     ANSIBLE_INSTALL_TIMEOUT = 500
@@ -107,6 +108,8 @@ class Config:
     ACTIVATE_XMLRPC = True
     FORCE_OIDC_AUTH = False
     BOOT_MODE = 0  # It can be 0-Normal, 1-ReadOnly, 2-ReadDelete
+    ENABLE_CORS = False
+    CORS_ORIGIN = '*'
 
 
 config = ConfigParser()
