@@ -18,6 +18,7 @@ import logging
 import operator
 import time
 import yaml
+import uuid
 
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.asymmetric import rsa
@@ -327,6 +328,38 @@ class CloudConnector(LoggerMixin):
         """
         raise NotImplementedError("Should have implemented this")
 
+    def list_images(self, auth_data, filters=None):
+        """
+        Get a list of images on the cloud provider using IM URI format.
+
+        Arguments:
+          - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+          - filters(:py:class:`dict` of str objects): Pair key value to filter the list of images.
+                                                     It is cloud provider specific.
+
+        Returns: a list dicts with at least two fields "uri" and "name".
+        """
+        raise NotImplementedError("Should have implemented this")
+
+    def get_quotas(self, auth_data):
+        """
+        Get the number of used and available resources in the cloud provider
+
+        Arguments:
+          - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+
+        Returns: dict with the following structure (if there are no limit in some metric, value is set to 1):
+                    {
+                        "cores": {"used": 1, "limit": 10},
+                        "ram": {"used": 1, "limit": 10},
+                        "instances": {"used": 1, "limit": 10},
+                        "floating_ips": {"used": 1, "limit": 10},
+                        "security_groups": {"used": 1, "limit": 10}
+                    }
+                 Some providers may provide more elements.
+        """
+        raise NotImplementedError("Should have implemented this")
+
     @staticmethod
     def keygen():
         """
@@ -424,7 +457,7 @@ class CloudConnector(LoggerMixin):
             return None
 
     @staticmethod
-    def get_instance_tags(system):
+    def get_instance_tags(system, auth_data=None, inf=None):
         """
         Get the instance_tags value of the system object as a dict
         """
@@ -436,6 +469,12 @@ class CloudConnector(LoggerMixin):
                 key = parts[0].strip()
                 value = parts[1].strip()
                 tags[key] = value
+        # If available try to set the IM username as a tag
+        if auth_data and auth_data.getAuthInfo('InfrastructureManager'):
+            im_username = auth_data.getAuthInfo('InfrastructureManager')[0]['username']
+            tags["IM-USER"] = im_username
+        if inf:
+            tags["IM_INFRA_ID"] = inf.id
         return tags
 
     @staticmethod
@@ -548,3 +587,42 @@ class CloudConnector(LoggerMixin):
                 return cidr
 
         return None
+
+    @staticmethod
+    def gen_instance_name(system, unique=True, default="im-userimage"):
+        name = system.getValue("instance_name")
+        if not name:
+            name = system.getValue("disk.0.image.name")
+        if not name:
+            name = system.name
+        if not name:
+            name = default
+        name = name.lower().replace("_", "-")
+        if unique:
+            return "%s-%s" % (name, str(uuid.uuid1()))
+        else:
+            return name
+
+    @staticmethod
+    def get_dns_entries(vm):
+        """
+        Get the required entries in the to add in the Cloud provider DNS service
+
+        Arguments:
+           - vm(:py:class:`IM.VirtualMachine`): VM information.
+        Returns: List of triplets (record, domain, ip)
+        """
+        res = []
+        system = vm.info.systems[0]
+        for net_name in system.getNetworkIDs():
+            num_conn = system.getNumNetworkWithConnection(net_name)
+            ip = system.getIfaceIP(num_conn)
+            (hostname, domain) = vm.getRequestedNameIface(num_conn,
+                                                          default_hostname=Config.DEFAULT_VM_NAME,
+                                                          default_domain=Config.DEFAULT_DOMAIN)
+            if domain != "localdomain" and ip and hostname:
+                if not domain.endswith("."):
+                    domain += "."
+                res.append((hostname, domain, ip))
+
+        return res
