@@ -53,10 +53,12 @@ class InstanceTypeInfo:
             - disk_space(int, optional): size of the disks
             - vpc_only(bool, optional): the instance works only on VPC
             - gpu(int, optional): the number of gpus of this instance
+            - gpu_model(str, optional): the model of the gpus of this instance
     """
 
     def __init__(self, name="", cpu_arch=None, num_cpu=1, cores_per_cpu=1, mem=0,
-                 price=0, cpu_perf=0, disks=0, disk_space=0, vpc_only=None, gpu=0):
+                 price=0, cpu_perf=0, disks=0, disk_space=0, vpc_only=None, gpu=0,
+                 gpu_model=None):
         self.name = name
         self.num_cpu = num_cpu
         self.cores_per_cpu = cores_per_cpu
@@ -70,6 +72,7 @@ class InstanceTypeInfo:
         self.disk_space = disk_space
         self.vpc_only = vpc_only
         self.gpu = gpu
+        self.gpu_model = gpu_model
 
 
 class EC2CloudConnector(CloudConnector):
@@ -136,23 +139,26 @@ class EC2CloudConnector(CloudConnector):
         """
         Update the features of the system with the information of the instance_type
         """
-        system.addFeature(Feature("cpu.count", "=", instance_type.num_cpu *
-                                  instance_type.cores_per_cpu), conflict="other", missing="other")
-        system.addFeature(Feature(
-            "memory.size", "=", instance_type.mem, 'M'), conflict="other", missing="other")
+        system.addFeature(Feature("cpu.count", "=", instance_type.num_cpu * instance_type.cores_per_cpu),
+                          conflict="other", missing="other")
+        system.addFeature(Feature("memory.size", "=", instance_type.mem, 'M'),
+                          conflict="other", missing="other")
         if instance_type.disks > 0:
-            system.addFeature(Feature("disks.free_size", "=", instance_type.disks *
-                                      instance_type.disk_space, 'G'), conflict="other", missing="other")
+            system.addFeature(Feature("disks.free_size", "=", instance_type.disks * instance_type.disk_space, 'G'),
+                              conflict="other", missing="other")
             for i in range(1, instance_type.disks + 1):
-                system.addFeature(Feature("disk.%d.free_size" % i, "=",
-                                          instance_type.disk_space, 'G'), conflict="other", missing="other")
-        system.addFeature(Feature("cpu.performance", "=",
-                                  instance_type.cpu_perf, 'ECU'), conflict="other", missing="other")
-        system.addFeature(
-            Feature("price", "=", instance_type.price), conflict="me", missing="other")
+                system.addFeature(Feature("disk.%d.free_size" % i, "=", instance_type.disk_space, 'G'),
+                                  conflict="other", missing="other")
+        system.addFeature(Feature("cpu.performance", "=", instance_type.cpu_perf, 'ECU'),
+                          conflict="other", missing="other")
+        system.addFeature(Feature("price", "=", instance_type.price), conflict="me", missing="other")
 
-        system.addFeature(Feature("instance_type", "=",
-                                  instance_type.name), conflict="other", missing="other")
+        system.addFeature(Feature("instance_type", "=", instance_type.name), conflict="other", missing="other")
+
+        if instance_type.gpu:
+            system.addFeature(Feature("gpu.count", "=", instance_type.gpu), conflict="other", missing="other")
+        if instance_type.gpu_model:
+            system.addFeature(Feature("gpu.model", "=", instance_type.gpu_model), conflict="other", missing="other")
 
     # Get the EC2 connection object
     def get_connection(self, region_name, auth_data):
@@ -266,6 +272,9 @@ class EC2CloudConnector(CloudConnector):
 
         (cpu, cpu_op, memory, memory_op, disk_free, disk_free_op) = self.get_instance_selectors(radl, disk_unit="G")
         arch = radl.getValue('cpu.arch', 'x86_64')
+        gpu = radl.getValue('gpu.count')
+        gpu_model = radl.getValue('gpu.model')
+        gpu_vendor = radl.getValue('gpu.vendor')
 
         performance = 0
         performance_op_str = ">="
@@ -291,6 +300,15 @@ class EC2CloudConnector(CloudConnector):
                 comparison = comparison and memory_op(instace_type.mem, memory)
                 comparison = comparison and disk_free_op(instace_type.disks * instace_type.disk_space, disk_free)
                 comparison = comparison and performance_op(instace_type.cpu_perf, performance)
+                if gpu and instace_type.gpu < gpu:
+                    continue
+
+                if gpu_model and (not instace_type.gpu_model or
+                                  gpu_model.lower() not in instace_type.gpu_model.lower()):
+                    continue
+                if gpu_vendor and (not instace_type.gpu_model or
+                                   gpu_vendor.lower() not in instace_type.gpu_model.lower()):
+                    continue
 
                 if comparison:
                     if not instance_type_name or instace_type.name == instance_type_name:
@@ -419,7 +437,7 @@ class EC2CloudConnector(CloudConnector):
         subnet_id = None
 
         for vpc in conn.get_all_vpcs():
-            if vpc.is_default:
+            if vpc.is_default or vpc.tags['Name'] == "default":
                 vpc_id = vpc.id
                 for subnet in conn.get_all_subnets(filters={"vpcId": vpc_id}):
                     subnet_id = subnet.id
@@ -580,10 +598,6 @@ class EC2CloudConnector(CloudConnector):
 
     def launch(self, inf, radl, requested_radl, num_vm, auth_data):
 
-        im_username = "im_user"
-        if auth_data.getAuthInfo('InfrastructureManager'):
-            im_username = auth_data.getAuthInfo('InfrastructureManager')[0]['username']
-
         system = radl.systems[0]
         placement = system.getValue('availability_zone')
 
@@ -676,7 +690,7 @@ class EC2CloudConnector(CloudConnector):
             user = self.DEFAULT_USER
             system.setValue('disk.0.os.credentials.username', user)
 
-        tags = self.get_instance_tags(system)
+        tags = self.get_instance_tags(system, auth_data, inf)
 
         all_failed = True
         volumes = {}
@@ -766,7 +780,7 @@ class EC2CloudConnector(CloudConnector):
                     if len(reservation.instances) == 1:
                         time.sleep(1)
                         instance = reservation.instances[0]
-                        instance.add_tag("IM-USER", im_username)
+                        instance.add_tag("Name", self.gen_instance_name(system))
                         for key, value in tags.items():
                             instance.add_tag(key, value)
                         ec2_vm_id = region_name + ";" + instance.id
@@ -1218,18 +1232,10 @@ class EC2CloudConnector(CloudConnector):
            - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
         """
         try:
-            region = vm.id.split(";")[0]
-            conn = self.get_route53_connection(region, auth_data)
-            system = vm.info.systems[0]
-            for net_name in system.getNetworkIDs():
-                num_conn = system.getNumNetworkWithConnection(net_name)
-                ip = system.getIfaceIP(num_conn)
-                (hostname, domain) = vm.getRequestedNameIface(num_conn,
-                                                              default_hostname=Config.DEFAULT_VM_NAME,
-                                                              default_domain=Config.DEFAULT_DOMAIN)
-                if domain != "localdomain" and ip:
-                    if not domain.endswith("."):
-                        domain += "."
+            dns_entries = self.get_dns_entries(vm)
+            if dns_entries:
+                conn = self.get_route53_connection('universal', auth_data)
+                for hostname, domain, ip in dns_entries:
                     zone = conn.get_zone(domain)
                     if not zone:
                         self.log_info("Creating DNS zone %s" % domain)
@@ -1262,18 +1268,10 @@ class EC2CloudConnector(CloudConnector):
            - vm(:py:class:`IM.VirtualMachine`): VM information.
            - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
         """
-        region = vm.id.split(";")[0]
-        conn = self.get_route53_connection(region, auth_data)
-        system = vm.info.systems[0]
-        for net_name in system.getNetworkIDs():
-            num_conn = system.getNumNetworkWithConnection(net_name)
-            ip = system.getIfaceIP(num_conn)
-            (hostname, domain) = vm.getRequestedNameIface(num_conn,
-                                                          default_hostname=Config.DEFAULT_VM_NAME,
-                                                          default_domain=Config.DEFAULT_DOMAIN)
-            if domain != "localdomain" and ip:
-                if not domain.endswith("."):
-                    domain += "."
+        dns_entries = self.get_dns_entries(vm)
+        if dns_entries:
+            conn = self.get_route53_connection('universal', auth_data)
+            for hostname, domain, ip in dns_entries:
                 zone = conn.get_zone(domain)
                 if not zone:
                     self.log_info("The DNS zone %s does not exists. Do not delete records." % domain)
@@ -1600,7 +1598,8 @@ class EC2CloudConnector(CloudConnector):
                                                       disks=disks,
                                                       disk_space=disk_space,
                                                       vpc_only=instance_type['vpc_only'],
-                                                      gpu=instance_type['GPU']))
+                                                      gpu=instance_type['GPU'],
+                                                      gpu_model=instance_type['GPU_model']))
             EC2CloudConnector.instance_type_list = instance_list
 
         return instance_list
@@ -1676,3 +1675,34 @@ class EC2CloudConnector(CloudConnector):
             return (True, "")
         else:
             return (False, "Error deregistering AMI image" + image_url)
+
+    def list_images(self, auth_data, filters=None):
+        regions = None
+        auth = auth_data.getAuthInfo(self.type)[0]
+        if 'region' in auth and auth['region']:
+            regions = [auth['region']]
+        if filters and 'region' in filters and filters['region']:
+            regions = [filters['region']]
+            del filters['region']
+        if not regions:
+            regions = [region.name for region in boto.ec2.regions()]
+
+        images_filter = {'architecture': 'x86_64', 'image-type': 'machine',
+                         'virtualization-type': 'hvm', 'state': 'available',
+                         'root-device-type': 'ebs'}
+
+        # enable the user to add or overwrite the filters
+        if filters:
+            images_filter.update(filters)
+
+        images = []
+        for region in regions:
+            conn = self.get_connection(region, auth_data)
+            try:
+                for image in conn.get_all_images(owners=['self', 'aws-marketplace'], filters=images_filter):
+                    if len(image.id) > 12:  # do not add old images
+                        images.append({"uri": "aws://%s/%s" % (region, image.id),
+                                       "name": "%s/%s" % (region, image.name)})
+            except Exception:
+                continue
+        return images
