@@ -983,7 +983,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         return nets
 
     @staticmethod
-    def get_volumes(driver, image, radl):
+    def get_volumes(driver, image, volume, radl):
         """
         Create the required volumes (in the RADL) for the VM.
 
@@ -994,18 +994,19 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         boot_disk = {
             'boot_index': 0,
-            'source_type': "image",
+            'source_type': "image" if image else "volume",
             'delete_on_termination': False,
-            'uuid': image.id
+            'uuid': image.id if image else volume.id
         }
 
+        if volume:
+            boot_disk['destination_type'] = 'volume'
         # if the user sets the size, we create a volume with this size
         # instead of booting directly from the image
-        if system.getValue("disk.0.size"):
+        elif system.getValue("disk.0.size"):
             boot_disk['destination_type'] = 'volume'
             boot_disk['volume_size'] = system.getFeature("disk.0.size").getValue('G')
             boot_disk['delete_on_termination'] = True
-            del boot_disk['device_name']
 
         res = [boot_disk]
 
@@ -1023,13 +1024,13 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
             disk_size = None
             if disk_url:
-                volume = driver.ex_get_volume(os.path.basename(disk_url))
+                new_volume = driver.ex_get_volume(os.path.basename(disk_url))
                 disk = {
                     'boot_index': cont,
                     'source_type': "volume",
                     'delete_on_termination': False,
                     'destination_type': "volume",
-                    'uuid': volume.id
+                    'uuid': new_volume.id
                 }
             else:
                 disk_size = system.getFeature("disk." + str(cont) + ".size").getValue('G')
@@ -1042,7 +1043,9 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     'delete_on_termination': True,
                     'volume_size': disk_size
                 }
-                if disk_type:
+                if disk_type == "ephemeral":
+                    disk['destination_type'] = 'local'
+                elif disk_type:
                     disk['volume_type'] = disk_type
             if disk_device:
                 disk['device_name'] = disk_device
@@ -1060,11 +1063,28 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 return auth['vo']
         return None
 
+    @staticmethod
+    def get_image_id(path):
+        url_path = urlparse(path)[2][1:]
+        if url_path.startswith("image"):
+            return url_path[6:]
+        elif not url_path.startswith("volume"):
+            return url_path
+        return None
+
+    @staticmethod
+    def get_volume_id(path):
+        url_path = urlparse(path)[2][1:]
+        if url_path.startswith("volume"):
+            return url_path[7:]
+        return None
+
     def launch(self, inf, radl, requested_radl, num_vm, auth_data):
         driver = self.get_driver(auth_data)
 
         system = radl.systems[0]
 
+        volume = image = None
         image_url = system.getValue("disk.0.image.url")
         if urlparse(image_url)[0] == "appdb":
             vo = self.get_vo_name(auth_data)
@@ -1074,13 +1094,22 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 raise Exception("Error in appdb image: %s" % msg)
         else:
             image_id = self.get_image_id(system.getValue("disk.0.image.url"))
-        image = driver.get_image(image_id)
+
+        if image_id:
+            image = driver.get_image(image_id)
+        else:
+            volume_id = self.get_volume_id(system.getValue("disk.0.image.url"))
+            if not volume_id:
+                raise Exception("Incorrect OpenStack image set.")
+            volume = driver.ex_get_volume(volume_id)
+            if not volume:
+                raise Exception("Incorrect OpenStack volume id.")
 
         instance_type = self.get_instance_type(driver, system)
         if not instance_type:
             raise Exception("No flavor found for the specified VM requirements.")
 
-        blockdevicemappings = self.get_volumes(driver, image, radl)
+        blockdevicemappings = self.get_volumes(driver, image, volume, radl)
 
         with inf._lock:
             self.create_networks(driver, radl, inf)
@@ -1844,6 +1873,10 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             net_quotas = driver.ex_get_network_quotas(tenant_id)
         except Exception:
             net_quotas = None
+        try:
+            vol_quotas = driver.ex_get_volume_quotas(tenant_id)
+        except Exception:
+            vol_quotas = None
 
         quotas_dict = {}
         quotas_dict["cores"] = {"used": quotas.cores.in_use + quotas.cores.reserved,
@@ -1863,4 +1896,10 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             quotas_dict["security_groups"] = {"used": net_quotas.security_group.in_use +
                                               net_quotas.security_group.reserved,
                                               "limit": net_quotas.security_group.limit}
+        if vol_quotas:
+            quotas_dict["volumes"] = {"used": vol_quotas.volumes.in_use + vol_quotas.volumes.reserved,
+                                      "limit": vol_quotas.volumes.limit}
+            quotas_dict["volume_storage"] = {"used": vol_quotas.gigabytes.in_use + vol_quotas.gigabytes.reserved,
+                                             "limit": vol_quotas.gigabytes.limit}
+
         return quotas_dict
